@@ -26,7 +26,12 @@
 #include <utility>
 #include <vector>
 
-#include "absl/log/check.h"
+#include "src/core/lib/security/authorization/audit_logging.h"
+#include "src/core/util/grpc_check.h"
+#include "src/core/util/json/json.h"
+#include "src/core/util/json/json_reader.h"
+#include "src/core/util/matchers.h"
+#include "src/core/util/useful.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/match.h"
@@ -34,11 +39,6 @@
 #include "absl/strings/str_format.h"
 #include "absl/strings/string_view.h"
 #include "absl/strings/strip.h"
-#include "src/core/lib/security/authorization/audit_logging.h"
-#include "src/core/util/json/json.h"
-#include "src/core/util/json/json_reader.h"
-#include "src/core/util/matchers.h"
-#include "src/core/util/useful.h"
 
 namespace grpc_core {
 
@@ -274,8 +274,8 @@ absl::StatusOr<Rbac::Permission> ParseRequest(const Json& json) {
 
 absl::StatusOr<Rbac::Policy> ParseRule(const Json& json,
                                        std::string* policy_name) {
-  absl::optional<Rbac::Principal> principals;
-  absl::optional<Rbac::Permission> permissions;
+  std::optional<Rbac::Principal> principals;
+  std::optional<Rbac::Permission> permissions;
   for (const auto& object : json.object()) {
     if (object.first == "name") {
       if (object.second.type() != Json::Type::kString) {
@@ -419,7 +419,7 @@ ParseAuditLogger(const Json& json, size_t pos) {
 }
 
 absl::Status ParseAuditLoggingOptions(const Json& json, RbacPolicies* rbacs) {
-  CHECK_NE(rbacs, nullptr);
+  GRPC_CHECK_NE(rbacs, nullptr);
   for (auto it = json.object().begin(); it != json.object().end(); ++it) {
     if (it->first == "audit_condition") {
       if (it->second.type() != Json::Type::kString) {
@@ -473,7 +473,7 @@ absl::Status ParseAuditLoggingOptions(const Json& json, RbacPolicies* rbacs) {
             // Parse again since it returns unique_ptr, but result should be ok
             // this time.
             auto result = ParseAuditLogger(loggers.at(i), i);
-            CHECK(result.ok());
+            GRPC_CHECK(result.ok());
             rbacs->deny_policy->logger_configs.push_back(
                 std::move(result.value()));
           }
