@@ -24,18 +24,17 @@
 #include <string>
 #include <utility>
 
-#include "absl/strings/string_view.h"
-
 #include "src/core/lib/channel/channel_args.h"
-#include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/iomgr/iomgr_fwd.h"
 #include "src/core/resolver/resolver.h"
 #include "src/core/resolver/resolver_factory.h"
-#include "src/core/lib/uri/uri_parser.h"
+#include "src/core/util/orphanable.h"
+#include "src/core/util/uri.h"
+#include "absl/strings/string_view.h"
 
 namespace grpc_core {
 
-class ResolverRegistry {
+class ResolverRegistry final {
  private:
   // Forward declaration needed to use this in Builder.
   struct State {
@@ -47,7 +46,7 @@ class ResolverRegistry {
   /// Methods used to create and populate the ResolverRegistry.
   /// NOT THREAD SAFE -- to be used only during global gRPC
   /// initialization and shutdown.
-  class Builder {
+  class Builder final {
    public:
     Builder();
 
