@@ -1231,7 +1231,7 @@ class ServicerContext(RpcContext, metaclass=abc.ABCMeta):
     def abort(self, code, details):
         """Raises an exception to terminate the RPC with a non-OK status.
 
-        The code and details passed as arguments will supercede any existing
+        The code and details passed as arguments will supersede any existing
         ones.
 
         Args:
@@ -1250,7 +1250,7 @@ class ServicerContext(RpcContext, metaclass=abc.ABCMeta):
     def abort_with_status(self, status):
         """Raises an exception to terminate the RPC with a non-OK status.
 
-        The status passed as argument will supercede any existing status code,
+        The status passed as argument will supersede any existing status code,
         status message and trailing metadata.
 
         This is an EXPERIMENTAL API.
@@ -1454,6 +1454,22 @@ class Server(abc.ABC):
         """
         raise NotImplementedError()
 
+    def add_registered_method_handlers(  # noqa: B027
+        self, service_name, method_handlers
+    ):
+        """Registers GenericRpcHandlers with this Server.
+
+        This method is only safe to call before the server is started.
+
+        If the same method have both generic and registered handler,
+        registered handler will take precedence.
+
+        Args:
+          service_name: The service name.
+          method_handlers: A dictionary that maps method names to corresponding
+            RpcMethodHandler.
+        """
+
     @abc.abstractmethod
     def add_insecure_port(self, address):
         """Opens an insecure port for accepting RPCs.
@@ -1824,25 +1840,23 @@ def ssl_server_credentials(
       object is an argument to add_secure_port() method during server setup.
     """
     if not private_key_certificate_chain_pairs:
-        raise ValueError(
+        error_msg = (
             "At least one private key-certificate chain pair is required!"
         )
-    elif require_client_auth and root_certificates is None:
-        raise ValueError(
-            "Illegal to require client auth without providing root"
-            " certificates!"
-        )
-    else:
-        return ServerCredentials(
-            _cygrpc.server_credentials_ssl(
-                root_certificates,
-                [
-                    _cygrpc.SslPemKeyCertPair(key, pem)
-                    for key, pem in private_key_certificate_chain_pairs
-                ],
-                require_client_auth,
-            )
+        raise ValueError(error_msg)
+    if require_client_auth and root_certificates is None:
+        error_msg = "Illegal to require client auth without providing root certificates!"
+        raise ValueError(error_msg)
+    return ServerCredentials(
+        _cygrpc.server_credentials_ssl(
+            root_certificates,
+            [
+                _cygrpc.SslPemKeyCertPair(key, pem)
+                for key, pem in private_key_certificate_chain_pairs
+            ],
+            require_client_auth,
         )
+    )
 
 
 def xds_server_credentials(fallback_credentials):
@@ -1895,10 +1909,8 @@ def ssl_server_certificate_configuration(
                 ],
             )
         )
-    else:
-        raise ValueError(
-            "At least one private key-certificate chain pair is required!"
-        )
+    error_msg = "At least one private key-certificate chain pair is required!"
+    raise ValueError(error_msg)
 
 
 def dynamic_ssl_server_credentials(
@@ -2024,6 +2036,7 @@ def alts_channel_credentials(service_accounts=None):
         peer identity of the server, handshake will fail. The arg can be empty
         if the client does not have any information about trusted server
         identity.
+
     Returns:
       A ChannelCredentials for use with an ALTS-enabled Channel
     """
@@ -2239,70 +2252,70 @@ class Compression(enum.IntEnum):
 ###################################  __all__  #################################
 
 __all__ = (
-    "FutureTimeoutError",
-    "FutureCancelledError",
-    "Future",
-    "ChannelConnectivity",
-    "StatusCode",
-    "Status",
-    "RpcError",
-    "RpcContext",
-    "Call",
-    "ChannelCredentials",
-    "CallCredentials",
     "AuthMetadataContext",
-    "AuthMetadataPluginCallback",
     "AuthMetadataPlugin",
-    "Compression",
+    "AuthMetadataPluginCallback",
+    "Call",
+    "CallCredentials",
+    "Channel",
+    "ChannelConnectivity",
+    "ChannelCredentials",
     "ClientCallDetails",
+    "Compression",
+    "Future",
+    "FutureCancelledError",
+    "FutureTimeoutError",
+    "GenericRpcHandler",
+    "HandlerCallDetails",
+    "LocalConnectionType",
+    "RpcContext",
+    "RpcError",
+    "RpcMethodHandler",
+    "Server",
     "ServerCertificateConfiguration",
     "ServerCredentials",
-    "LocalConnectionType",
-    "UnaryUnaryMultiCallable",
-    "UnaryStreamMultiCallable",
-    "StreamUnaryMultiCallable",
-    "StreamStreamMultiCallable",
-    "UnaryUnaryClientInterceptor",
-    "UnaryStreamClientInterceptor",
-    "StreamUnaryClientInterceptor",
-    "StreamStreamClientInterceptor",
-    "Channel",
-    "ServicerContext",
-    "RpcMethodHandler",
-    "HandlerCallDetails",
-    "GenericRpcHandler",
-    "ServiceRpcHandler",
-    "Server",
     "ServerInterceptor",
-    "unary_unary_rpc_method_handler",
-    "unary_stream_rpc_method_handler",
-    "stream_unary_rpc_method_handler",
-    "stream_stream_rpc_method_handler",
-    "method_handlers_generic_handler",
-    "ssl_channel_credentials",
-    "metadata_call_credentials",
+    "ServiceRpcHandler",
+    "ServicerContext",
+    "Status",
+    "StatusCode",
+    "StreamStreamClientInterceptor",
+    "StreamStreamMultiCallable",
+    "StreamUnaryClientInterceptor",
+    "StreamUnaryMultiCallable",
+    "UnaryStreamClientInterceptor",
+    "UnaryStreamMultiCallable",
+    "UnaryUnaryClientInterceptor",
+    "UnaryUnaryMultiCallable",
     "access_token_call_credentials",
+    "alts_channel_credentials",
+    "alts_server_credentials",
+    "channel_ready_future",
     "composite_call_credentials",
     "composite_channel_credentials",
     "compute_engine_channel_credentials",
+    "dynamic_ssl_server_credentials",
+    "insecure_channel",
+    "insecure_server_credentials",
+    "intercept_channel",
     "local_channel_credentials",
     "local_server_credentials",
-    "alts_channel_credentials",
-    "alts_server_credentials",
-    "ssl_server_credentials",
-    "ssl_server_certificate_configuration",
-    "dynamic_ssl_server_credentials",
-    "channel_ready_future",
-    "insecure_channel",
-    "secure_channel",
-    "intercept_channel",
-    "server",
+    "metadata_call_credentials",
+    "method_handlers_generic_handler",
     "protos",
-    "services",
     "protos_and_services",
+    "secure_channel",
+    "server",
+    "services",
+    "ssl_channel_credentials",
+    "ssl_server_certificate_configuration",
+    "ssl_server_credentials",
+    "stream_stream_rpc_method_handler",
+    "stream_unary_rpc_method_handler",
+    "unary_stream_rpc_method_handler",
+    "unary_unary_rpc_method_handler",
     "xds_channel_credentials",
     "xds_server_credentials",
-    "insecure_server_credentials",
 )
 
 ############################### Extension Shims ################################
